#!/bin/bash
#
# EeePC Actions
# Andrew Wyatt
# Generic script to take actions on keypress
#

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
  . $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh
EEE_USER=$(who | sed -n '/ (:0[\.0]*)$\| :0 /{s/ .*//p;q}')

SELECTION=$3
if [ "$KEY_SHOW" = "1" ]; then
  eeepc_notify "You pressed: \"$SELECTION\"" keyboard
fi
case $SELECTION in
  $KEY_RESOLUTION)
     su $EEE_USER -l -c "DISPLAY=:0 $EEEPC_PATH/eeepc-lvds-resolution-toggle.sh toggle" &
  ;;
  $KEY_ROTATE)
     su $EEE_USER -l -c "DISPLAY=:0 $EEEPC_PATH/eeepc-rotate-lvds.sh" &
  ;;
  $KEY_FSB)
     $EEEPC_PATH/eeepc-cpu-control.sh &
  ;;
  $KEY_VOLU)
     if [ "$VOL_DEV" = "AUTO" ]; then
       VOL_DEVS=($(amixer -c 0 controls | awk -F "," ' $_ ~ /Playback Volume/ \
	{split($3,a,"=");print a[2]}' | sed -s -e "s#\sPlayback\sVolume'##" -e "s#^'##"))

       for dev in ${VOL_DEVS[*]}; do
         amixer -q -c 0 set $dev $VOL_ADJ+ >/dev/null 2>&1
       done
     elif [ "$VOL_DEV" = "FAKE" ]; then
       acpi_fakekey 115
     else
       amixer -q -c 0 set $VOL_DEV $VOL_ADJ+ &
     fi
  ;;
  $KEY_VOLD)
     if [ "$VOL_DEV" = "AUTO" ]; then
       VOL_DEVS=($(amixer -c 0 controls | awk -F "," ' $_ ~ /Playback Volume/ \
	{split($3,a,"=");print a[2]}' | sed -s -e "s#\sPlayback\sVolume'##" -e "s#^'##"))

       for dev in ${VOL_DEVS[*]}; do
         amixer -q -c 0 set $dev $VOL_ADJ- >/dev/null 2>&1
       done
     elif [ "$VOL_DEV" = "FAKE" ]; then
       acpi_fakekey 114
     else
       amixer -q -c 0 set $VOL_DEV $VOL_ADJ- &
     fi
  ;;
  $KEY_MUTE)
     if [ "$VOL_DEV" = "AUTO" ]; then
       MUTE_DEVS=($(amixer -c 0 controls | grep "Playback Switch" | awk -F "," '{printf $1" "}'))
       MUTE_STATE=$(amixer -c 0 cget ${MUTE_DEVS[0]},iface=MIXER | awk '$_ ~ /:/ {split($2,a,"="); print a[2]}')
       if [[ "$MUTE_STATE" =~ "on,on" ]]; then
         for dev in ${MUTE_DEVS[*]}; do
           amixer -q -c 0 cset $dev,iface=MIXER 0 >/dev/null 2>&1
         done
       else
         for dev in ${MUTE_DEVS[*]}; do
           amixer -q -c 0 cset $dev,iface=MIXER 1 >/dev/null 2>&1
         done
       fi
     fi
  ;;
  $KEY_VGAOFF)
     $EEEPC_PATH/eeepc-lvds-toggle.sh &
  ;;
  $KEY_VGAOUTA)
     $EEEPC_PATH/eeepc-vga-toggle.sh &
  ;;
  $KEY_VGAOUTB)
     $EEEPC_PATH/eeepc-vga-toggle.sh &
  ;;
  $KEY_VGAOUTC)
     $EEEPC_PATH/eeepc-vga-toggle.sh &
  ;;
  $KEY_PERFMON)
    eeepc_notify "Launching $KEY_PERFMON_NAME" "$KEY_PERFMON_ICON"
    su $EEE_USER -l -c "DISPLAY=:0 $KEY_PERFMON_COMMAND $KEY_PERFMON_ICON" &
  ;;
  $KEY_TOUCHPAD)
    $EEEPC_PATH/eeepc-touchpad-toggle.sh &
  ;;
  $KEY_WIFI)
     $EEEPC_PATH/eeepc-wifi-toggle.sh &
  ;;
  $KEY_BT)
     $EEEPC_PATH/eeepc-bluetooth-toggle.sh &
  ;;
  $KEY_CAM)
     $EEEPC_PATH/eeepc-camera-toggle.sh &
  ;;
  $KEY_CUSTOMA)
    eeepc_notify "Launching $KEY_CUSTOMA_NAME" "$KEY_CUSTOMA_ICON"
    su $EEE_USER -l -c "DISPLAY=:0 $KEY_CUSTOMA_COMMAND" &
  ;;
  $KEY_CUSTOMB)
    eeepc_notify "Launching $KEY_CUSTOMB_NAME" "$KEY_CUSTOMB_ICON"
    su $EEE_USER -l -c "DISPLAY=:0 $KEY_CUSTOMB_COMMAND" &
  ;;
  $KEY_CUSTOMC)
    eeepc_notify "Launching $KEY_CUSTOMC_NAME" "$KEY_CUSTOMC_ICON"
    su $EEE_USER -l -c "DISPLAY=:0 $KEY_CUSTOMC_COMMAND" &
  ;;
  $KEY_CUSTOMD)
    eeepc_notify "Launching $KEY_CUSTOMD_NAME" "$KEY_CUSTOMD_ICON"
    su $EEE_USER -l -c "DISPLAY=:0 $KEY_CUSTOMD_COMMAND" &
  ;;
esac
