#!/bin/bash
#
# EeePC Camera toggle
# Andrew Wyatt
# Tool to toggle Camera device on / off

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
  . $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh

NICON="camera-web"

if [ -e "/sys/devices/platform/eeepc/camera" ]; then
  CAMERA_DEVICE="/sys/devices/platform/eeepc/camera"
fi

if [ ! "$CAMERA_DEVICE" ] && [ -e "/proc/acpi/asus/camera" ]; then
  CAMERA_DEVICE="/proc/acpi/asus/camera"
fi

if [ ! "$CAMERA_DEVICE" ]; then
  CAMERA_STATUS="-1"
else
  CAMERA_STATUS=$(cat $CAMERA_DEVICE)
fi

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR 2>/dev/null
fi

if [ -e "$EEEPC_VAR/cam_saved" ]; then
  CAMERA_SAVED=$(cat $EEEPC_VAR/cam_saved)
else
  CAMERA_SAVED=1
fi

function cam_status {
  if [ "$CAMERA_STATUS" = "-1" ] || [ "$CAMERA_STATUS" = "0" ]; then
    echo OFF
  else
    echo ON
  fi
}

function check_cam_installed {
  if [ "$CAMERA_STATUS" = "-1" ]; then
    eeepc_notify "Webcam module not installed" $NICON
    exit 1
  fi
}

function cam_disable {
  check_cam_installed
  eeepc_notify "Webcam Disabled" $NICON
  echo 0 > $EEEPC_VAR/cam_saved
  echo 0 > $CAMERA_DEVICE
  /sbin/modprobe -r uvcvideo
}

function cam_poweroff {
  echo 0 > $CAMERA_DEVICE
}

function cam_enable {
  check_cam_installed
  eeepc_notify "Webcam Enabled" $NICON
  /sbin/modprobe uvcvideo 2>/dev/null
  echo 1 > $EEEPC_VAR/cam_saved
  echo 1 > $CAMERA_DEVICE
}

function cam_toggle {
  check_cam_installed
  if [ "$CAMERA_STATUS" = "1" ]; then
    cam_disable
  elif [ "$CAMERA_STATUS" = "0" ]; then
    cam_enable
  fi
}

function cam_restore {
  if [ "$CAMERA_SAVED" = "1" ]; then
    if [ "$CAMERA_STATUS" = "0" ]; then
      # eeepc_notify "Webcam Enabled" $NICON
      /sbin/modprobe uvcvideo
      echo 1 > $CAMERA_DEVICE
    fi
  elif [ "$CAMERA_SAVED" = "0" ]; then
    if [ "$CAMERA_STATUS" = "1" ]; then
      # eeepc_notify "Webcam Disabled" $NICON
      echo 0 > $CAMERA_DEVICE
      /sbin/modprobe -r uvcvideo 2>/dev/null
    fi
  fi
}

case $1 in
  restore)
    cam_restore
  ;;
  status)
    cam_status
  ;;
  on|enable)
    cam_enable
  ;;
  off|disable)
    cam_disable
  ;;
  poweroff)
    cam_poweroff
  ;;
  *)
    cam_toggle
  ;;
esac
