#!/bin/bash
# 
# EeePC CPU Front Side Bus control utility
# Andrew Wyatt
# Generic FSB control utility to shift between modes
#

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
  . $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh
NICON="gnome-cpu-frequency-applet"

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR 2>/dev/null
fi

function cleanup {
  if [ -e "$EEEPC_VAR/eeepc-cpu-control-busy" ]; then
    rm -f $EEEPC_VAR/eeepc-cpu-control-busy 2>/dev/null
  fi
}

if [ -e "$EEEPC_VAR/eeepc-cpu-control-busy" ]; then
  countdown="2"
  while [ $countdown -gt 0 ]; do
    sleep 1
    if [ ! -e "$EEEPC_VAR/eeepc-cpu-control-busy" ]; then
      break
    fi
    countdown=$[$countdown-1]
  done
  rm -f $EEEPC_VAR/eeepc-cpu-control-busy
else
  touch "$EEEPC_VAR/eeepc-cpu-control-busy"
fi

if [ -e "/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor" ]; then
  FSB_CPU0=$(cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor)
fi

if [ -e "/sys/devices/system/cpu/cpu1/cpufreq/scaling_governor" ]; then
  FSB_CPU1=$(cat /sys/devices/system/cpu/cpu1/cpufreq/scaling_governor)
fi

FSB_CONTROL="/proc/netbook/bus_control"

FSB_DEFAULT_CONTROL="$EEEPC_VAR/default_fsb"

if [ -e "/sys/devices/platform/eeepc/she" ]; then
  SHE_DEVICE="/sys/devices/platform/eeepc/she"
elif [ -e "/sys/devices/platform/eeepc/cpufv" ]; then
  SHE_DEVICE="/sys/devices/platform/eeepc/cpufv"
fi

if [ ! -e "$SHE_DEVICE" ]; then
  BUS_CONTROL="UNKNOWN"
fi

if [ -e "$FSB_DEFAULT_CONTROL" ]; then
  FSB_DEFAULT_STATE=$(cat $FSB_DEFAULT_CONTROL)
  FSB_DYNAMIC_A=$(echo $FSB_DEFAULT_STATE | awk '{printf $1}')
  FSB_DYNAMIC_B=$(echo $FSB_DEFAULT_STATE | awk '{printf $2}')
  FSB_PCIE=$(echo $FSB_DEFAULT_STATE | awk '{printf $3}')
else
  if [ -e "$SHE_DEVICE" ]; then
    BUS_MODE="SHE"
  else
    DONOT_SCALE_FSB=1
  fi
fi

INSANE_BUS=$OVERCLOCK_SETTING

# 1000HD, 904HD
if [ "$FSB_DYNAMIC_A" = "15" ]; then
  if [ "$FSB_DYNAMIC_B" = "30" ]; then
    SUPER_BUS="143 30 15 1"
    HIGH_BUS="15 30 143 0"
    POWERSAVE_BUS="15 24 143 0"
  fi
fi

# 901
if [ "$FSB_DYNAMIC_A" = "50" ]; then
  if [ "$FSB_DYNAMIC_B" = "105" ]; then
    SUPER_BUS="50 105 207 1"
    HIGH_BUS="50 100 207 0"
    POWERSAVE_BUS="50 75 207 0"
  fi
fi

# 901GO, 1000, 1000HE
if [ "$FSB_DYNAMIC_A" = "50" ]; then
  if [ "$FSB_DYNAMIC_B" = "100" ]; then
    SUPER_BUS="50 105 15 1"
    HIGH_BUS="50 100 207 0"
    POWERSAVE_BUS="50 75 207 0"
  fi
fi

if [ "$FSB_DYNAMIC_A" = "24" ]; then
  # 700 series
  if [ "$FSB_DYNAMIC_B" = "70" ]; then
    SUPER_BUS="24 80 00 1"
    HIGH_BUS="24 70 00 0"
    POWERSAVE_BUS="24 64 00 0"
  fi
  # 900HD 900
  if [ "$FSB_DYNAMIC_B" = "100" ]; then
    SUPER_BUS="24 100 00 1"
    HIGH_BUS="24 85 00 0"
    POWERSAVE_BUS="24 70 00 0"
  fi
fi

if [ ! "$SUPER_SETTING" = "" ]; then
  SUPER_BUS=$SUPER_SETTING
fi

if [ ! "$HIGH_SETTING" = "" ]; then
  HIGH_BUS=$HIGH_SETTING
fi

if [ ! "$POWERSAVE_SETTING" = "" ]; then
  POWERSAVE_BUS=$POWERSAVE_SETTING
fi

if [ "$FSB_DYNAMIC_A" = "0" ]; then
  if [ -e "$SHE_DEVICE" ]; then
    BUS_MODE="SHE"
  else
    DONOT_SCALE_FSB=1
  fi
fi

if [ -e "$EEEPC_VAR/cpu_mode" ]; then
  FSB_SAVED=$(cat $EEEPC_VAR/cpu_mode)
else
  FSB_SAVED=$FSB_CPU0
fi

function set_bus_mode {
  BUS=$1
  if [ ! "$DONOT_SCALE_FSB" = "1" ]; then
    echo $BUS >$FSB_CONTROL
  fi
}

function legacy_scale {

  if [ ! "$FSB_SCALE" ]; then
    FSB_SCALE="10"
  fi

  if [ "$DONOT_SCALE_FSB" = "1" ]; then
    return 0
  fi

  FSB_NATIVE=$(echo $FSB_DEFAULT_STATE | awk '{print $2}')
  FSB_CURRENT=$(cat $FSB_CONTROL)
  FSB_CURRENT_A=$(echo $FSB_CURRENT | awk '{printf $1}')
  FSB_CURRENT_B=$(echo $FSB_CURRENT | awk '{printf $2}')
  FSB_SCALE=$(echo "scale=2; $FSB_SCALE/100" | bc)
  FSB_SCALED=$(echo "$FSB_NATIVE-($FSB_NATIVE*$FSB_SCALE)/1" | bc)

  if [ "$1" = "insane" ]; then
    SCALE_VOLTAGE=1
    SCALE_PATH_A=$FSB_CURRENT_B
    SCALE_PATH_B=$OVERCLOCK_SETTING
  elif [ "$1" = "super" ]; then
    SCALE_VOLTAGE=1
    SCALE_PATH_A=$FSB_CURRENT_B
    SCALE_PATH_B=$FSB_NATIVE
  elif [ "$1" = "high" ]; then
    SCALE_VOLTAGE=0
    SCALE_PATH_A=$FSB_CURRENT_B
    SCALE_PATH_B=$FSB_NATIVE
  elif [ "$1" = "powersave" ]; then
    SCALE_PATH_A=$FSB_DYNAMIC_B
    SCALE_PATH_B=$FSB_SCALED
    SCALE_VOLTAGE=0
  else
    cleanup
    exit 0
  fi

  if (( "$SCALE_PATH_A" > "$SCALE_PATH_B" )); then
    SCALE_DIRECTION="-1"
  else
    SCALE_DIRECTION="1"
  fi
  if [ "$SCALE_PATH_B" = "$FSB_CURRENT_B" ] || [ "$SCALE_PATH_A" = "$FSB_CURRENT_A" ]; then
    cleanup
    exit 0
  fi

  for i in $(seq $SCALE_PATH_A $SCALE_DIRECTION $SCALE_PATH_B); do
    echo "$FSB_DYNAMIC_A $i $FSB_PCIE $SCALE_VOLTAGE" >$FSB_CONTROL
    sleep 0.5
  done
}

function fsb_mode {
  FSBTARGET=$1

  if [ "$FSBTARGET" = "insane" ]; then
    if [ ! "$BUS_MODE" = "SHE" ]; then
      if [ "$OVERCLOCK_SETTING" ]; then
        RMODE="Insane Performance"
        CPUMODE="performance"
        if [ "$INSANE_BUS" ]; then
          set_bus_mode "$SUPER_BUS"
	  sleep 0.2
          set_bus_mode "$INSANE_BUS"
        else
          legacy_scale insane
        fi
      else
        FSBTARGET="super"
      fi
    else
      FSBTARGET="super"
    fi
  fi

  if [ "$FSBTARGET" = "super" ]; then
    RMODE="Super High Performance"
    CPUMODE="performance"
    if [ "$BUS_MODE" = "SHE" ]; then
      echo "0" > $SHE_DEVICE
    else
      if [ ! "$BUS_MODE" = "LEGACY" ] && [ "$SUPER_BUS" ]; then
        set_bus_mode "$SUPER_BUS"
      else
        legacy_scale super
      fi
    fi
  fi

  if [ "$FSBTARGET" = "high" ]; then
    RMODE="High Performance"
    CPUMODE="ondemand"
    if [ "$BUS_MODE" = "SHE" ]; then
      echo "1" > $SHE_DEVICE
    else
      if [ ! "$BUS_MODE" = "LEGACY" ] && [ "$HIGH_BUS" ]; then
        set_bus_mode "$HIGH_BUS"
      else
        legacy_scale high
      fi
    fi
  fi
  
  if [ "$FSBTARGET" = "powersave" ]; then
    RMODE="Power Saver"
    CPUMODE="powersave"
    if [ "$BUS_MODE" = "SHE" ]; then
      echo "2" > $SHE_DEVICE
    else
      if [ ! "$BUS_MODE" = "LEGACY" ] && [ "$POWERSAVE_BUS" ]; then
        set_bus_mode "$POWERSAVE_BUS"
      else
        legacy_scale powersave
      fi
    fi
  fi

  if [ ! "$CPUMODE" ]; then
    CPUMODE="$FSBTARGET"
  fi

  if [ "$FASTER_POWERSAVE" = "1" ]; then
    if [ "$FSBTARGET" = "powersave" ]; then
      CPUMODE="ondemand"
    fi
  fi

  if [ -e "/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor" ]; then
    echo $CPUMODE > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
  fi

  if [ -e "/sys/devices/system/cpu/cpu1/cpufreq/scaling_governor" ]; then
    echo $CPUMODE > /sys/devices/system/cpu/cpu1/cpufreq/scaling_governor
  fi

  eeepc_notify "$RMODE Mode" $NICON

  if [ ! "$CPU_MODE_SCRIPTS" = "0" ]; then
    if [ -d "$EEEPC_PATH/$FSBTARGET.d" ]; then
      for perf_item in $EEEPC_PATH/$FSBTARGET.d/*
      do
        $perf_item > /dev/null 2>&1 || true
      done
    fi
  fi

  if [ ! "$FSBTARGET" = "default" ]; then
    echo $FSBTARGET >$EEEPC_VAR/cpu_mode
  fi
}

function fsb_toggle {
  if [ "$FSB_SAVED" = "insane" ]; then
    fsb_mode "super"
  elif [ "$FSB_SAVED" = "super" ]; then
    fsb_mode "high"
  elif [ "$FSB_SAVED" = "high" ]; then
    fsb_mode "powersave"
  elif [ "$FSB_SAVED" = "powersave" ]; then
    fsb_mode "insane"
  fi
}

function cpu_debug {
    DBGOUT="/tmp/cpu_debug.txt"
    echo "Outputting to: [$DBGOUT]"
    echo "Please generate and sent this file to the maintainer whenever reporting CPU issues."
    echo "Eee PC CPU: [$EEEPC_CPU]" >>$DBGOUT
    echo "FSB Control Device: [$FSB_CONTROL]" >>$DBGOUT
    echo "CURRENT FSB: [$FSB_CURRENT]" >>$DBGOUT
    echo "DETECTED Dynamic Overclock A: [$FSB_DYNAMIC_A]" >>$DBGOUT
    echo "DETECTED Dynamic Overclock B: [$FSB_DYNAMIC_B]" >>$DBGOUT
    echo "-------------------------------------------------------------------------------" >>$DBGOUT
    if [ -e "/proc/netbook/bus_control" ]; then
      echo "Front Side Bus:" >>$DBGOUT
      cat /proc/netbook/bus_control  >>$DBGOUT
    else
      echo "NOTE: FSB unavailable (no /proc/netbook/bus_control)"
    fi
    echo "-------------------------------------------------------------------------------" >>$DBGOUT
    echo "CPU:" >>$DBGOUT
    cat /proc/cpuinfo >>$DBGOUT
    echo "-------------------------------------------------------------------------------" >>$DBGOUT
    echo "DMI:" >>$DBGOUT
    dmidecode >>$DBGOUT
    echo "-------------------------------------------------------------------------------" >>$DBGOUT
    if [ -e "/sys/devices/system/cpu/cpu0/cpufreq" ]; then
      cd /sys/devices/system/cpu/cpu0/cpufreq
      for i in *
      do
        echo "$i: [$(cat $i 2>/dev/null)]" >>$DBGOUT 2>/dev/null
      done
    else
      echo "NOTE: CPUFreq not available"
    fi
    echo "-------------------------------------------------------------------------------" >>$DBGOUT
    cleanup
}

case $1 in
  restore)
    NO_NOTIFY=1
    fsb_mode $FSB_SAVED
  ;;
  super)
    fsb_mode "super"
  ;;
  insane)
    fsb_mode "insane"
  ;;
  high)
    fsb_mode "high"
  ;;
  powersave)
    fsb_mode "powersave"
  ;;
  default)
    fsb_mode "default"
  ;;
  debug)
    cpu_debug
  ;;
  *)
    fsb_toggle
  ;;
esac
cleanup
