#!/usr/bin/perl
$|=1;
use POSIX qw(setsid);

if (-e "/var/run/eeefan") {
  open (in,"</var/run/eeefan");
   $pgrp=<in>;
   chomp $pgrp;
  close (in);
  $check=`(ps -fp $pgrp 2>&1 >/dev/null && echo ACTIVE) || echo DEAD`;
  chomp $check;
  if ($check =~/ACTIVE/) {
    exit 0;
  } else {
    syslog_send("Found dead Eee Fan process, resuming");
    unlink ("/var/run/eeefan");
  }
}

if ( -e "/proc/netbook/temperature" ) {
  $SYS{'FAN_SPEED_CONTROL'}="/proc/netbook/fan_speed";
  $SYS{'FAN_SWITCH'}="/proc/netbook/fan_control";
  $SYS{'TEMP_CONTROL'}="/proc/netbook/temperature";
} else {
  syslog_send("CRITICAL: EEEPC-FAN-CONTROL UNABLE TO MANAGE FAN");
  exit 1;
}

sub syslog_send {
  $msg=$_[0];
  if ( -e "/usr/bin/logger") {
    system("/usr/bin/logger -p user.info \"$msg\"");
  } else {
    print "WARNING: UNABLE TO USE SYSLOG\n"
  }
}

sub get_config {
  $query=$_[0];
  if ( -e "/var/eeepc/hardware" ) {
    open(in,"</var/eeepc/hardware");
    while (<in>) {
      chomp;
      unless ( $_ =~/^#/ || $_ !~/[A-z]/gi) {
        ($key,$value)=split(/\=/,$_);
        $value=~s/\"//g;
        if ($key =~/$query/) {
          close(in);
          return $value;
        }
      }
    }
  close (in);
  }
  if ( -e "/etc/default/eeepc-acpi.local" ) {
    open(in,"</etc/default/eeepc-acpi.local");
    while (<in>) {
      chomp;
      unless ( $_ =~/^#/ || $_ !~/[A-z]/gi) {
        ($key,$value)=split(/\=/,$_);
        $value=~s/\"//g;
        if ($key =~/$query/) {
          close(in);
          return $value;
        }
      }
    }
  close (in);
  }
  if ( -e "/etc/default/eeepc-acpi" ) {
    open(in,"</etc/default/eeepc-acpi");
    while (<in>) {
      chomp;
      unless ( $_ =~/^#/ || $_ !~/[A-z]/gi) {
        ($key,$value)=split(/\=/,$_);
        $value=~s/\"//g;
        if ($key =~/$query/) {
          close(in);
	  return $value;
        }
      }
    }
  close (in);
  return "";
  } else {
    die "Could not read configuration file";
  }
}

sub read_ctrl {
  $toread=$_[0];
  if (-e "$toread") {
    open(in,"<$toread");
     $toread=<in>;
     chomp $toread;
    close(in);
    return $toread;
  }
}

sub set_fan_mode {
  $mode=$_[0];
  open(in,"<$SYS{'FAN_SWITCH'}");
   $tmpmod=<in>;
   chomp $tmpmod;
  close(in);
  unless ($mode eq $tmpmod) {
    if ($mode eq "0") {
      $nmode="AUTO";
    } else {
      $nmode="MANAGED";
    }
    syslog_send("NOTICE: FAN MODE CHANGED TO $nmode");
    open (out,">$SYS{'FAN_SWITCH'}");
      print out $mode."\n";
    close (out);
  }
}

sub set_fan_speed {
  $speed=$_[0];
  open (in,"<$SYS{'FAN_SPEED_CONTROL'}");
   $tmpspd=<in>;
  chomp $tmpspd;
  close (in);
  unless ($tmpspd eq $speed) {
    open (out,">$SYS{'FAN_SPEED_CONTROL'}");
     print out $speed."\n";
    close out;
  }
}

chdir '/';
umask 0;
open (STDIN,'/dev/null');
open (STDERR,'>/dev/null');
open (STDOUT,'>/dev/null');
defined($pid=fork) || die "Can not fork..";
exit if $pid;
setsid;
$PRCS=$$;
open (out,">/var/run/eeefan");
print out $PRCS;
close (out);

for (;;) {
  $Eee{'SWITCH'}=read_ctrl($SYS{'FAN_SWITCH'});
  $Eee{'TEMP'}=read_ctrl($SYS{'TEMP_CONTROL'});
  $Eee{'FAN_SPEED'}=read_ctrl($SYS{'FAN_SPEED_CONTROL'});
  
  $Eee{'FAN_MODE'} = get_config("FAN_MODE");
  $Eee{'TEMP_NORMAL'} = get_config("TEMP_NORMAL");
  $Eee{'TEMP_WARN'} = get_config("TEMP_WARN");
  $Eee{'TEMP_CRIT'} = get_config("TEMP_CRIT");

  $Eee{'MAX_FAN_SPEED'}=get_config("MAX_FAN_SPEED");
  $Eee{'NOM_FAN_SPEED'}=get_config("NOM_FAN_SPEED");
  $Eee{'SUB_FAN_SPEED'}=get_config("SUB_FAN_SPEED");
  $Eee{'MIN_FAN_SPEED'}=get_config("MIN_FAN_SPEED");

  if (! $Eee{'MAX_FAN_SPEED'}) {
    $Eee{'MAX_FAN_SPEED'}="100";
  }

  $Eee{'EEEPC_CPU'}=get_config("EEEPC_CPU");
  if (! $Eee{"NOM_FAN_SPEED"}) {
    $Eee{"NOM_FAN_SPEED"}=sprintf("%02d",$Eee{'MAX_FAN_SPEED'}/2);
  }

  if (! $Eee{"SUB_FAN_SPEED"}) {
    if ( $Eee{'EEEPC_CPU'} =~/Celeron/i) {
      $Eee{"SUB_FAN_SPEED"}=sprintf("%02d",$Eee{'MAX_FAN_SPEED'}/3);
    } else {
      $Eee{"SUB_FAN_SPEED"}=sprintf("%02d",$Eee{'MAX_FAN_SPEED'}/4);
    }
  }

  if (! $Eee{"MIN_FAN_SPEED"}) {
    if ( $Eee{'EEEPC_CPU'} =~/Celeron/i) {
      $Eee{"MIN_FAN_SPEED"}=sprintf("%02d",$Eee{'MAX_FAN_SPEED'}/4);
    } else {
      $Eee{"MIN_FAN_SPEED"}=sprintf("%02d",$Eee{'MAX_FAN_SPEED'}/6);
    }
  }

  if ($Eee{'FAN_MODE'} =~/DISABLED/gi) {
    syslog_send("NOTICE: Eee PC FAN Control disabled.");
    set_fan_mode("0");
    exit 0;
  }

  if ($Eee{'FAN_MODE'} =~/MANAGED/gi) {
    set_fan_mode("1");

    if ($Eee{'TEMP'} ge $Eee{'TEMP_NORMAL'}) {
      set_fan_speed($Eee{"SUB_FAN_SPEED"});
    } elsif ($Eee{'TEMP'} le $Eee{'TEMP_NORMAL'} - 2) {
        set_fan_speed($Eee{"MIN_FAN_SPEED"});
    }

    if ( $Eee{'TEMP'} ge $Eee{'TEMP_WARN'} ) {
      if ( $Eee{'TEMP'} ge $Eee{'TEMP_CRIT'} ) {
        syslog_send("CRITICAL: Eee PC heat dangerously high. [$Eee{'TEMP'}]");
        set_fan_speed($Eee{'MAX_FAN_SPEED'});
      } elsif ( $Eee{'TEMP'} le $Eee{'TEMP_CRIT'} - 2) {
        set_fan_speed($Eee{'NOM_FAN_SPEED'});
      }
   }

  } else {
    set_fan_mode("0");
  }
  sleep 3;
}
