#!/bin/bash
#
# EeePC LVDS resolution toggle
# Andrew Wyatt
# Tool to toggle LVDS output resolutions

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
. $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh

NICON="display"

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR
fi

LVDS_DEVICE=$(xrandr | grep " connected" | awk '{print $1}' | grep LVDS)
LVDS_MODES=$(xrandr | grep -F -e $LVDS_DEVICE -A 12 | grep [0-9]x[0-9] | grep -v -e LVDS | awk '{printf $1" "}')
  if [[ "$LVDS_MODES" =~ "1024x600" ]]; then
    LVDS_MODES="1024x768 $LVDS_MODES"
  elif [[ "$LVDS_MODES" =~ "800x480" ]]; then
    LVDS_MODES="800x600 $LVDS_MODES"
  fi

LVDS_CURRENT=$(xrandr | grep -F -e $LVDS_DEVICE | awk '{print $3}' | sed -e "s#\+.*##")
NEW_RESOLUTION=$1;

function save_modes {
    SAVE_LVDS_MODES=$(xrandr | grep -F -e $LVDS_DEVICE -A 12 | grep [0-9]x[0-9] | grep -v -e LVDS | awk '{printf $1" "}')
    #SAVE_LVDS_CURRENT=$(xrandr | grep -F -e $LVDS_DEVICE -A 12 | grep -F -e [0-9]x[0-9] -e "*" | awk '{print $1}')
    SAVE_LVDS_CURRENT=$(xrandr | grep -F -e $LVDS_DEVICE | awk '{print $3}' | sed -e "s#\+.*##")
    if [[ "$SAVE_LVDS_MODES" =~ "1024x600" ]]; then
      SAVE_LVDS_MODES="1024x768 $SAVE_LVDS_MODES"
    elif [[ "$SAVE_LVDS_MODES" =~ "800x480" ]]; then
      SAVE_LVDS_MODES="800x600 $SAVE_LVDS_MODES"
    fi
    echo -n "$SAVE_LVDS_CURRENT $SAVE_LVDS_MODES" > $EEEPC_VAR/available_resolutions
}

function toggle_resolution {
  for MODE in $LVDS_MODES $LVDS_MODES; do
    if [ "$MODE" = "$LVDS_CURRENT" ]; then
      NEXT=1;
    elif [ "$NEXT" = "1" ]; then
      if [ "$MODE" = "1024x768" ]; then
        if [[ "$LVDS_MODES" =~ "1024x600" ]]; then
          CHMODE="1024x600"
          SCALE="1.00x1.28"
        fi
      elif [ "$MODE" = "800x600" ]; then
        if [[ "$LVDS_MODES" =~ "800x480" ]]; then
          CHMODE="800x480"
          SCALE="1.00x1.25"
        fi
      fi
      if [ "$CHMODE" = "" ]; then
        CHMODE=$MODE
        SCALE="1.00x1.00"
      fi
      xrandr --output $LVDS_DEVICE --mode $CHMODE --scale $SCALE
      eeepc_notify "LVDS Resolution: \"$MODE\"" $NICON
      echo $MODE > $EEEPC_VAR/resolution_saved
      save_modes
      exit
    fi
  done
  exit
}

function set_resolution {
  if [ "$1" = "" ]; then
    toggle_resolution
    exit
  fi
  MODE=$1
  if [ "$MODE" = "1024x768" ]; then
    if [[ "$LVDS_MODES" =~ "1024x600" ]]; then
      CHMODE="1024x600"
      SCALE="1.00x1.28"
    fi
  elif [ "$MODE" = "800x600" ]; then
    if [[ "$LVDS_MODES" =~ "800x480" ]]; then
      CHMODE="800x480"
      SCALE="1.00x1.25"
    fi
  fi
  if [ "$CHMODE" = "" ]; then
    CHMODE=$MODE
    SCALE="1.00x1.00"
  fi

  xrandr --output $LVDS_DEVICE --mode $CHMODE --scale $SCALE
  eeepc_notify "LVDS Resolution: \"$MODE\"" $NICON
  echo $MODE > $EEEPC_VAR/resolution_saved
  save_modes
  exit
}

function restore_resolution {
  if [ -e "$EEEPC_VAR/resolution_saved" ]; then
    MODE=$(cat $EEEPC_VAR/resolution_saved)
    if [ ! "$MODE" = "$LVDS_CURRENT" ]; then
      if [ "$MODE" = "1024x768" ]; then
        if [[ "$LVDS_MODES" =~ "1024x600" ]]; then
          CHMODE="1024x600"
          SCALE="1.00x1.28"
        fi
      elif [ "$MODE" = "800x600" ]; then
        if [[ "$LVDS_MODES" =~ "800x480" ]]; then
          CHMODE="800x480"
          SCALE="1.00x1.25"
        fi
      fi
      if [ "$CHMODE" = "" ]; then
        CHMODE=$MODE
        SCALE="1.00x1.00"
      fi

      xrandr --output $LVDS_DEVICE --mode $CHMODE --scale $SCALE
      # eeepc_notify "LVDS Resolution: \"$MODE\"" $NICON
    fi
  fi
  save_modes
  exit
}

case $1 in
  restore)
    restore_resolution
  ;;
  toggle)
    toggle_resolution
  ;;
  modes)
    save_modes
  ;;
  *)
    set_resolution $NEW_RESOLUTION
  ;;
esac
