#!/bin/bash
#
# LVDS Rotate
# Andrew Wyatt
# Tool to rotate LVDS panel
#

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
. $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh

NICON="display"

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR
fi

LVDS_DEVICE=$(xrandr | grep " connected" | awk '{print $1}' | grep LVDS)

function rotate_toggle {
  CURRENT=$(xrandr  | grep $LVDS_DEVICE | awk '{print $4}' | sed s#^\(##)
  if [ "$CURRENT" = "left" ]; then
    ROTATION="inverted"
    RTOUCHPAD=3
  elif [ "$CURRENT" = "inverted" ]; then
    ROTATION="right"
    RTOUCHPAD=1
  elif [ "$CURRENT" = "right" ]; then
    ROTATION="normal"
    RTOUCHPAD=0
  else
    ROTATION="left"
    RTOUCHPAD=2
  fi
  if [ "$ROTATION" ]; then
    echo "$ROTATION $RTOUCHPAD" > $EEEPC_VAR/rotation_saved
    rotate_lvds $ROTATION $RTOUCHPAD
  fi
}

function restore_rotation {
  if [ -e "$EEEPC_VAR/rotation_saved" ]; then
    CURRENT=$(xrandr  | grep $LVDS_DEVICE | awk '{print $4}' | sed s#^\(##)
    RMODES=$(cat $EEEPC_VAR/rotation_saved)
    ROTATION=$(echo $RMODES | awk '{print $1}')
    RTOUCHPAD=$(echo $RMODES | awk '{print $2}')
    if [ ! "$CURRENT" = "$ROTATION" ]; then
      NO_NOTIFY=1
      rotate_lvds $ROTATION $RTOUCHPAD
    fi
  fi
}

function rotate_lvds {
    eeepc_notify "Rotate LCD \"$1\"" $NICON
    xrandr --output $LVDS_DEVICE --rotate "$1"
    synclient Xrandr=$2
    echo "$1 $2" > $EEEPC_VAR/rotation_saved
}

case $1 in
  inverted)
    rotate_lvds inverted 3
  ;;
  left)
    rotate_lvds left 2
  ;;
  right)
    rotate_lvds right 1
  ;;
  normal)
    rotate_lvds normal 0
  ;;
  restore)
    restore_rotation
  ;;
  *)
    rotate_toggle
  ;;
esac
