#!/bin/bash
#
# EeePC Touchpad toggle
# Andrew Wyatt
# Tool to enable / disable touchpad
#
# Syndaemon idea and concept provided by luigidk and bixejo, thank you.
#

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
. $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh

NICON="input-tablet"

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR
fi

if [ -e "$EEEPC_VAR/touchpad_saved" ]; then
  TPSAVED=$(cat $EEEPC_VAR/touchpad_saved)
fi

function touchpad_status {
  TPSTATE=$(synclient -l 2>/dev/null | grep TouchpadOff | awk '{printf $3}')
  if [ "$TPSTATE" = "" ]; then
    TPSTATE=2
  fi
  if [ -e "$EEEPC_VAR/syndaemon-eeepc" ] || [ "$TPSTATE" = 0 ]; then
    echo "ON"
  elif [ "$TPSTATE" = "2" ]; then
    echo "UNAVAILABLE"
  else
    echo "OFF"
  fi
}

function touchpad_toggle {
  TOUCHPAD=$(synclient -l | grep TouchpadOff | awk '{printf $3}')
  if [ -e "$EEEPC_VAR/syndaemon-eeepc" ] || [ "$TOUCHPAD" = "0" ]; then
    kill $(cat $EEEPC_VAR/syndaemon-eeepc)
    rm -f $EEEPC_VAR/syndaemon-eeepc >/dev/null 2>&1 || true
    echo 0 > $EEEPC_VAR/touchpad_saved
    su $EEE_USER -l -c "DISPLAY=:0 synclient TouchpadOff=1"
    if [ $? = "0" ]; then
      eeepc_notify "Touchpad Disabled" $NICON
    else
      eeepc_notify "Unable to disable touchpad; Ensure xorg.conf is properly configured." $NICON
    fi
  else
    echo 1 > $EEEPC_VAR/touchpad_saved
    su $EEE_USER -l -c "DISPLAY=:0 synclient TouchpadOff=0"
    if [ $? = "0" ]; then
      if [ "$TOUCHPAD_KEY_DISABLE"  = "1" ]; then
        su $EEE_USER -l -c "DISPLAY=:0 syndaemon -i $TOUCHPAD_DELAY -d -p $EEEPC_VAR/syndaemon-eeepc"
      fi
      eeepc_notify "Touchpad Enabled" $NICON
    else
      eeepc_notify "Unable to enable touchpad; Ensure xorg.conf is properly configured." $NICON
    fi
  fi
}

function touchpad_restore {
  if [ "$TPSAVED" = "0" ]; then
    synclient TouchpadOff=1
    if [ $? = "0" ]; then
      if [ -e "$EEEPC_VAR/syndaemon-eeepc" ]; then
        kill $(cat $EEEPC_VAR/syndaemon-eeepc)
      fi
    fi
  else
    synclient TouchpadOff=0
    if [ $? = "0" ]; then
      if [ "$TOUCHPAD_KEY_DISABLE" = "1" ]; then
        syndaemon -i $TOUCHPAD_DELAY -d -p $EEEPC_VAR/syndaemon-eeepc
      fi
    fi
  fi
}

case $1 in
  restore)
    touchpad_restore
  ;;
  status)
    touchpad_status
  ;;
  *)
    touchpad_toggle
  ;;
esac
