#!/bin/bash
#
# EeePC VGA toggle
# Andrew Wyatt
# Tool to enable / disable VGA output

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
. $EEEPC_VAR/hardware
fi

. /etc/acpi/eeepc/eeepc-notify.sh

NICON="display"

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR 2>/dev/null
fi

if [ -e "$EEEPC_VAR/vga_saved" ]; then
  VGA_RESTORE=$(cat $EEEPC_VAR/vga_saved)
fi

LVDS_DEVICE=$(xrandr | grep " connected" | awk '{print $1}' | grep LVDS)
VGA_DEVICE=$(xrandr | grep " connected" | awk '{print $1}' | grep VGA)

VGA_XRANDR=$(xrandr)
VGA_STATUS=$(echo "$VGA_XRANDR" | grep 'VGA connected' >/dev/null 2>&1 && echo connected)
VGA_ENABLED=$(echo "$VGA_XRANDR" | grep 'VGA connected' -A 1 | grep "\*" >/dev/null 2>&1 && echo 1)

function video_toggle {
  if [ "$VGA_RESTORE" = "lvds" ]; then
    vga
  elif [ "$VGA_RESTORE" = "vga" ]; then
    clone
  else
    lvds
  fi
}

function vga {
  if [ ! "$VGA_STATUS" = "connected" ]; then
      eeepc_notify "External monitor not detected." $NICON
      exit 0
  fi
  xrandr --output $VGA_DEVICE --auto
  if [ "$?" = "0" ]; then
    xrandr --output $LVDS_DEVICE --off
    eeepc_notify "External Display Mode" $NICON
    echo vga > $EEEPC_VAR/vga_saved
    $EEEPC_PATH/eeepc-lvds-resolution-toggle.sh modes
  else
    eeepc_notify "Could not enable monitor." $NICON
  fi
  exit 0
}

function clone {
  if [ ! "$VGA_STATUS" = "connected" ]; then
      eeepc_notify "External monitor not detected." $NICON
      exit 0
  fi
  xrandr --output $VGA_DEVICE --auto
  if [ "$?" = "0" ]; then
    xrandr --output $LVDS_DEVICe --auto
    eeepc_notify "Clone Display Mode" $NICON
    echo clone > $EEEPC_VAR/vga_saved
    $EEEPC_PATH/eeepc-lvds-resolution-toggle.sh modes
  else
    eeepc_notify "Could not enable monitor." $NICON
  fi
  exit 0
}

function lvds {
  xrandr --output $LVDS_DEVICE --auto
  if [ "$?" = "0" ]; then
    xrandr --output $VGA_DEVICE --off
    eeepc_notify "Internal Display Mode" $NICON
    echo lvds > $EEEPC_VAR/vga_saved
    $EEEPC_PATH/eeepc-lvds-resolution-toggle.sh modes
  else
    eeepc_notify "Could not enable monitor." $NICON
  fi
  exit 0 
}

case $1 in
  restore)
    NO_NOTIFY=1
    $VGA_RESTORE
  ;;
  clone)
    clone
  ;;
  vga)
    vga
  ;;
  lvds)
    lvds
  ;;
  *)
    video_toggle
  ;;
esac
