#!/bin/bash
#
#  EeePC WIFI Toggle
#  by Andrew Wyatt
#  Generic WIFI toggle utility which should work across EeePC models.
#

WIFI_MODULES="-e ath5k -e r8180 -e rtl8180 -e rtl8187 -e ath9k -e rt2860sta"

. /etc/default/eeepc-acpi
if [ -e "/etc/default/eeepc-acpi.local" ]; then
  . /etc/default/eeepc-acpi.local
fi

if [ -e "$EEEPC_VAR/hardware" ]; then
. $EEEPC_VAR/hardware
fi

WIFI_RFKILL=$(for i in /sys/class/rfkill/*; do (echo -n $i/state:;cat $i/type) | grep wlan | sed "s#:.*##"; done | head -n 1)

. /etc/acpi/eeepc/eeepc-notify.sh

NICON="gnome-dev-wavelan"

if [ -e $EEEPC_VAR/eeepc-wifi-saved ]; then
  RADIO_SAVED_STATE=$(cat $EEEPC_VAR/eeepc-wifi-saved)
else
  RADIO_SAVED_STATE=0
fi

RADIO_CONTROL=$WIFI_RFKILL
if [ -e "$RADIO_CONTROL" ]; then
  RADIO_STATE=$(cat $RADIO_CONTROL);
fi

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR 2>/dev/null
fi

WIFI_IF=$(iwconfig 2>/dev/null | grep ESSID | awk '{print $1}')

function wifi_status {
  if [ "$RADIO_STATE" = "-1" ]; then
    echo "UNKNOWN"
  elif [ "$RADIO_STATE" = "0" ]; then
    echo "OFF"
  else
    echo "ON"
  fi
}

function get_wifi_driver {
    if [ "$WIFI_DRIVER" = "" ]; then
      WIFI_DRIVER=$(lspci -k | grep $WIFI_MODULES | awk '{print $3}' | tail -n 1 | sed -s -e "s/\W$//")
      (grep WIFI_DRIVER $EEEPC_VAR/hardware >/dev/null 2>&1) || echo "WIFI_DRIVER=$WIFI_DRIVER" >> $EEEPC_VAR/hardware
    fi
}


function radio_on {
  echo 1 > $RADIO_CONTROL
  sleep 1
  get_wifi_driver
  /sbin/modprobe $WIFI_DRIVER 2>/dev/null
  WIFI_RFKILLB=$(for i in /sys/class/rfkill/*; do (echo -n $i/state:;cat $i/type) | grep wlan | sed "s#:.*##"; done | tail -n 1)
  echo 1 > $WIFI_RFKILLB || true

  if [ $(cat $RADIO_CONTROL 2>/dev/null) = "1" ]; then
    echo 1 > $EEEPC_VAR/eeepc-wifi-saved
    eeepc_notify "WIFI Radio on" $NICON
  else
    eeepc_notify "Could not enable WIFI radio." $NICON
  fi
}

function radio_off {
  get_wifi_driver
  /sbin/ifconfig $WIFI_IF down 2>/dev/null
  /sbin/rmmod $WIFI_DRIVER 2>/dev/null
  sleep 1
  echo 0 > $RADIO_CONTROL
  if [ $(cat $RADIO_CONTROL 2>/dev/null) = "0" ]; then
    echo 0 > $EEEPC_VAR/eeepc-wifi-saved
    eeepc_notify "WIFI Radio off" $NICON
  else
    eeepc_notify "Could not disable WIFI radio." $NICON
  fi
}

function radio_poweroff {
  get_wifi_driver
  /sbin/ifconfig $WIFI_IF down 2>/dev/null
  /sbin/rmmod $WIFI_DRIVER 2>/dev/null
  sleep 1
  echo 0 > $RADIO_CONTROL
}

function radio_toggle {
  if [ "$RADIO_STATE" = "1" ]; then
    radio_off
  else
    radio_on
  fi
}

function radio_restore {
  NO_NOTIFY=1
  if [ "$RADIO_SAVED_STATE" = "1" ]; then
    if [ "$RADIO_STATE" = "0" ]; then
      radio_on
    fi
  else
    if [ "$RADIO_STATE" = "1" ]; then
      radio_off
    fi
  fi
}

function wifi_debug {
  DBGOUT="/tmp/wifi_debug.txt"
  echo "Outputting to: [$DBGOUT]"
  echo "Please generate and sent this file to the maintainer whenever reporting WIFI issues."
  if [ $(cat $RADIO_CONTROL) = "0" ]; then
    echo "Please turn WIFI on before running if possible."
  fi
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "LSPCI" >>$DBGOUT
  lspci -v >>$DBGOUT || true
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "RADIO OFF:" >>$DBGOUT
  echo "[/sbin/ifconfig $WIFI_IF down]" >>$DBGOUT
  /sbin/ifconfig $WIFI_IF down >>$DBGOUT 2>&1
  get_wifi_driver
  echo "[/sbin/modprobe -r $WIFI_DRIVER]" >>$DBGOUT
  /sbin/modprobe -r $WIFI_DRIVER >>$DBGOUT 2>&1 || true
  echo "[echo 0 > $RADIO_CONTROL]" >>$DBGOUT
  echo 0 > $RADIO_CONTROL || true
  cat $RADIO_CONTROL >>$DBGOUT
  echo "[echo 0 > $EEEPC_VAR/eeepc-wifi-saved]" >>$DBGOUT
  echo 0 > $EEEPC_VAR/eeepc-wifi-saved
  cat $EEEPC_VAR/eeepc-wifi-saved >>$DBGOUT
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "ifconfig" >>$DBGOUT
  ifconfig -a >>$DBGOUT 2>&1
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "IWConfig" >>$DBGOUT
  iwconfig >>$DBGOUT 2>&1
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "LSMOD" >>$DBGOUT
  lsmod >>$DBGOUT 2>&1
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "RADIO ON:" >>$DBGOUT
  echo "[echo 1 > $RADIO_CONTROL]" >>$DBGOUT
  echo 1 > $RADIO_CONTROL
  cat $RADIO_CONTROL >>$DBGOUT || true
  get_wifi_driver
  echo "[/sbin/modprobe $WIFI_DRIVER]" >>$DBGOUT
  /sbin/modprobe $WIFI_DRIVER >>$DBGOUT 2>&1 || true
  echo "[echo 1 > $EEEPC_VAR/eeepc-wifi-saved]" >>$DBGOUT
  echo 1 > $EEEPC_VAR/eeepc-wifi-saved
  cat $EEEPC_VAR/eeepc-wifi-saved >>$DBGOUT
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "ifconfig" >>$DBGOUT
  ifconfig -a >>$DBGOUT 2>&1
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "IWConfig" >>$DBGOUT
  iwconfig >>$DBGOUT 2>&1
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
  echo "LSMOD" >>$DBGOUT
  lsmod >>$DBGOUT 2>&1
  echo "-------------------------------------------------------------------------------" >>$DBGOUT
}

case $1 in
  restore)
    radio_restore
  ;;
  status)
    wifi_status
  ;;
  on|enable)
    radio_on
  ;;
  off|disable)
    radio_off
  ;;
  poweroff)
    radio_poweroff
  ;;
  debug)
    wifi_debug
  ;;
  *)
    radio_toggle
  ;;
esac
